#ifndef _JIFT_H
#define _JIFT_H

#include <vil/vil_image_view.h>
#include <vector>
#include <string>
#include "Descriptor.h"
#include "Keypoint.h"


using namespace std;

// the main class for JIFT operation
class JIFT
{
  public:
    JIFT(vil_image_view<vxl_byte> const&, unsigned int, unsigned int);
    JIFT(string const& filename, unsigned int, unsigned int);
    JIFT(const char* filename, unsigned int, unsigned int);
    void init(vil_image_view<vxl_byte> const& srcimg);
    ~JIFT();
    void doJIFT(void);
    const vector<Descriptor> & descriptor() const;
    const vector<Keypoint>   & keypoint() const;
    const vil_image_view<vxl_byte> & srcimg()const;
    

    
  private:
    void buildPyramid(void);
    void detectLocalExtrema(void);
    void assignOrientation(void);
    void extractKeypointDescriptor(void);
    
  private:
        // the source image
    vil_image_view<vxl_byte> m_srcimg;
        // octaves and intervals, oriented from SIFT
    unsigned int m_octaves;
    unsigned int m_intervals;
    
        // gaussian pyramid
    vil_image_view<double> ** m_glist;
        // difference of gaussian pyramid
    vil_image_view<double> ** m_doglist;
        // extrema
    vil_image_view<bool>   ** m_extrema;
        // absolute sigma
    double ** m_abssigmas;
        // number of keypoints
    unsigned int m_kpnum;
    
    vector<Keypoint>   m_kps;
    vector<Descriptor> m_desc;
};

#endif
